@@ -1,17 +1,27 @@
 /*
-    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
 
-    This file is part of 0MQ.
+    This file is part of libzmq, the ZeroMQ core engine in C++.
 
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -20,7 +30,6 @@
 #include "platform.hpp"
 
 #ifdef HAVE_LIBSODIUM
-#include <sodium.h>
 
 #ifdef ZMQ_HAVE_WINDOWS
 #include "windows.hpp"
@@ -39,14 +48,25 @@ zmq::curve_server_t::curve_server_t (session_base_t *session_,
     session (session_),
     peer_address (peer_address_),
     state (expect_hello),
-    expecting_zap_reply (false),
-    cn_nonce (1)
+    cn_nonce (1),
+    cn_peer_nonce(1),
+    sync()
 {
+    int rc;
     //  Fetch our secret key from socket options
     memcpy (secret_key, options_.curve_secret_key, crypto_box_SECRETKEYBYTES);
+    scoped_lock_t lock (sync);
+#if defined(HAVE_TWEETNACL)
+    // allow opening of /dev/urandom
+    unsigned char tmpbytes[4];
+    randombytes(tmpbytes, 4);
+#else
+    rc = sodium_init ();
+    zmq_assert (rc != -1);
+#endif
 
     //  Generate short-term key pair
-    const int rc = crypto_box_keypair (cn_public, cn_secret);
+    rc = crypto_box_keypair (cn_public, cn_secret);
     zmq_assert (rc == 0);
 }
 
@@ -69,6 +89,11 @@ int zmq::curve_server_t::next_handshake_command (msg_t *msg_)
             if (rc == 0)
                 state = connected;
             break;
+        case send_error:
+            rc = produce_error (msg_);
+            if (rc == 0)
+                state = error_sent;
+            break;
         default:
             errno = EAGAIN;
             rc = -1;
@@ -84,15 +109,13 @@ int zmq::curve_server_t::process_handshake_command (msg_t *msg_)
     switch (state) {
         case expect_hello:
             rc = process_hello (msg_);
-            if (rc == 0)
-                state = send_welcome;
             break;
         case expect_initiate:
             rc = process_initiate (msg_);
-            if (rc == 0)
-                state = expecting_zap_reply? expect_zap_reply: send_ready;
             break;
         default:
+            //  Temporary support for security debugging
+            puts ("CURVE I: invalid handshake command");
             errno = EPROTO;
             rc = -1;
             break;
@@ -114,7 +137,7 @@ int zmq::curve_server_t::encode (msg_t *msg_)
 
     uint8_t message_nonce [crypto_box_NONCEBYTES];
     memcpy (message_nonce, "CurveZMQMESSAGES", 16);
-    memcpy (message_nonce + 16, &cn_nonce, 8);
+    put_uint64 (message_nonce + 16, cn_nonce);
 
     uint8_t flags = 0;
     if (msg_->flags () & msg_t::more)
@@ -144,7 +167,7 @@ int zmq::curve_server_t::encode (msg_t *msg_)
     uint8_t *message = static_cast <uint8_t *> (msg_->data ());
 
     memcpy (message, "\x07MESSAGE", 8);
-    memcpy (message + 8, &cn_nonce, 8);
+    memcpy (message + 8, message_nonce + 16, 8);
     memcpy (message + 16, message_box + crypto_box_BOXZEROBYTES,
             mlen - crypto_box_BOXZEROBYTES);
 
@@ -161,12 +184,16 @@ int zmq::curve_server_t::decode (msg_t *msg_)
     zmq_assert (state == connected);
 
     if (msg_->size () < 33) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: invalid CURVE client, sent malformed command");
         errno = EPROTO;
         return -1;
     }
 
     const uint8_t *message = static_cast <uint8_t *> (msg_->data ());
     if (memcmp (message, "\x07MESSAGE", 8)) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: invalid CURVE client, did not send MESSAGE");
         errno = EPROTO;
         return -1;
     }
@@ -174,6 +201,12 @@ int zmq::curve_server_t::decode (msg_t *msg_)
     uint8_t message_nonce [crypto_box_NONCEBYTES];
     memcpy (message_nonce, "CurveZMQMESSAGEC", 16);
     memcpy (message_nonce + 16, message + 8, 8);
+    uint64_t nonce = get_uint64(message + 8);
+    if (nonce <= cn_peer_nonce) {
+        errno = EPROTO;
+        return -1;
+    }
+    cn_peer_nonce = nonce;
 
     const size_t clen = crypto_box_BOXZEROBYTES + msg_->size () - 16;
 
@@ -204,9 +237,11 @@ int zmq::curve_server_t::decode (msg_t *msg_)
                 message_plaintext + crypto_box_ZEROBYTES + 1,
                 msg_->size ());
     }
-    else
+    else {
+        //  Temporary support for security debugging
+        puts ("CURVE I: connection key used for MESSAGE is wrong");
         errno = EPROTO;
-
+    }
     free (message_plaintext);
     free (message_box);
 
@@ -221,24 +256,36 @@ int zmq::curve_server_t::zap_msg_available ()
     }
     const int rc = receive_and_process_zap_reply ();
     if (rc == 0)
-        state = send_ready;
+        state = status_code == "200"
+            ? send_ready
+            : send_error;
     return rc;
 }
 
-bool zmq::curve_server_t::is_handshake_complete () const
+zmq::mechanism_t::status_t zmq::curve_server_t::status () const
 {
-    return state == connected;
+    if (state == connected)
+        return mechanism_t::ready;
+    else
+    if (state == error_sent)
+        return mechanism_t::error;
+    else
+        return mechanism_t::handshaking;
 }
 
 int zmq::curve_server_t::process_hello (msg_t *msg_)
 {
     if (msg_->size () != 200) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: client HELLO is not correct size");
         errno = EPROTO;
         return -1;
     }
 
     const uint8_t * const hello = static_cast <uint8_t *> (msg_->data ());
     if (memcmp (hello, "\x05HELLO", 6)) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: client HELLO has invalid command name");
         errno = EPROTO;
         return -1;
     }
@@ -247,6 +294,8 @@ int zmq::curve_server_t::process_hello (msg_t *msg_)
     const uint8_t minor = hello [7];
 
     if (major != 1 || minor != 0) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: client HELLO has unknown version number");
         errno = EPROTO;
         return -1;
     }
@@ -260,6 +309,7 @@ int zmq::curve_server_t::process_hello (msg_t *msg_)
 
     memcpy (hello_nonce, "CurveZMQHELLO---", 16);
     memcpy (hello_nonce + 16, hello + 112, 8);
+    cn_peer_nonce = get_uint64(hello + 112);
 
     memset (hello_box, 0, crypto_box_BOXZEROBYTES);
     memcpy (hello_box + crypto_box_BOXZEROBYTES, hello + 120, 80);
@@ -269,10 +319,13 @@ int zmq::curve_server_t::process_hello (msg_t *msg_)
                               sizeof hello_box,
                               hello_nonce, cn_client, secret_key);
     if (rc != 0) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: cannot open client HELLO -- wrong server key?");
         errno = EPROTO;
         return -1;
     }
 
+    state = send_welcome;
     return rc;
 }
 
@@ -339,12 +392,16 @@ int zmq::curve_server_t::produce_welcome (msg_t *msg_)
 int zmq::curve_server_t::process_initiate (msg_t *msg_)
 {
     if (msg_->size () < 257) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: client INITIATE is not correct size");
         errno = EPROTO;
         return -1;
     }
 
     const uint8_t *initiate = static_cast <uint8_t *> (msg_->data ());
     if (memcmp (initiate, "\x08INITIATE", 9)) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: client INITIATE has invalid command name");
         errno = EPROTO;
         return -1;
     }
@@ -364,6 +421,8 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
                                     sizeof cookie_box,
                                     cookie_nonce, cookie_key);
     if (rc != 0) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: cannot open client INITIATE cookie");
         errno = EPROTO;
         return -1;
     }
@@ -371,6 +430,8 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
     //  Check cookie plain text is as expected [C' + s']
     if (memcmp (cookie_plaintext + crypto_secretbox_ZEROBYTES, cn_client, 32)
     ||  memcmp (cookie_plaintext + crypto_secretbox_ZEROBYTES + 32, cn_secret, 32)) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: client INITIATE cookie is not valid");
         errno = EPROTO;
         return -1;
     }
@@ -388,10 +449,13 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
 
     memcpy (initiate_nonce, "CurveZMQINITIATE", 16);
     memcpy (initiate_nonce + 16, initiate + 105, 8);
+    cn_peer_nonce = get_uint64(initiate + 105);
 
     rc = crypto_box_open (initiate_plaintext, initiate_box,
                           clen, initiate_nonce, cn_client, cn_secret);
     if (rc != 0) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: cannot open client INITIATE");
         errno = EPROTO;
         return -1;
     }
@@ -415,12 +479,16 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
                           sizeof vouch_box,
                           vouch_nonce, client_key, cn_secret);
     if (rc != 0) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: cannot open client INITIATE vouch");
         errno = EPROTO;
         return -1;
     }
 
     //  What we decrypted must be the client's short-term public key
     if (memcmp (vouch_plaintext + crypto_box_ZEROBYTES, cn_client, 32)) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: invalid handshake from client (public key)");
         errno = EPROTO;
         return -1;
     }
@@ -434,12 +502,18 @@ int zmq::curve_server_t::process_initiate (msg_t *msg_)
     if (rc == 0) {
         send_zap_request (client_key);
         rc = receive_and_process_zap_reply ();
-        if (rc != 0) {
-            if (errno != EAGAIN)
-                return -1;
-            expecting_zap_reply = true;
-        }
+        if (rc == 0)
+            state = status_code == "200"
+                ? send_ready
+                : send_error;
+        else
+        if (errno == EAGAIN)
+            state = expect_zap_reply;
+        else
+            return -1;
     }
+    else
+        state = send_ready;
 
     return parse_metadata (initiate_plaintext + crypto_box_ZEROBYTES + 128,
                            clen - crypto_box_ZEROBYTES - 128);
@@ -463,13 +537,12 @@ int zmq::curve_server_t::produce_ready (msg_t *msg_)
     if (options.type == ZMQ_REQ
     ||  options.type == ZMQ_DEALER
     ||  options.type == ZMQ_ROUTER)
-        ptr += add_property (ptr, "Identity",
-            options.identity, options.identity_size);
+        ptr += add_property (ptr, "Identity", options.identity, options.identity_size);
 
     const size_t mlen = ptr - ready_plaintext;
 
     memcpy (ready_nonce, "CurveZMQREADY---", 16);
-    memcpy (ready_nonce + 16, &cn_nonce, 8);
+    put_uint64 (ready_nonce + 16, cn_nonce);
 
     int rc = crypto_box_afternm (ready_box, ready_plaintext,
                                  mlen, ready_nonce, cn_precom);
@@ -482,7 +555,7 @@ int zmq::curve_server_t::produce_ready (msg_t *msg_)
 
     memcpy (ready, "\x05READY", 6);
     //  Short nonce, prefixed by "CurveZMQREADY---"
-    memcpy (ready + 6, &cn_nonce, 8);
+    memcpy (ready + 6, ready_nonce + 16, 8);
     //  Box [metadata](S'->C')
     memcpy (ready + 14, ready_box + crypto_box_BOXZEROBYTES,
             mlen - crypto_box_BOXZEROBYTES);
@@ -492,6 +565,18 @@ int zmq::curve_server_t::produce_ready (msg_t *msg_)
     return 0;
 }
 
+int zmq::curve_server_t::produce_error (msg_t *msg_) const
+{
+    zmq_assert (status_code.length () == 3);
+    const int rc = msg_->init_size (6 + 1 + status_code.length ());
+    zmq_assert (rc == 0);
+    char *msg_data = static_cast <char *> (msg_->data ());
+    memcpy (msg_data, "\5ERROR", 6);
+    msg_data [6] = sizeof status_code;
+    memcpy (msg_data + 7, status_code.c_str (), status_code.length ());
+    return 0;
+}
+
 void zmq::curve_server_t::send_zap_request (const uint8_t *key)
 {
     int rc;
@@ -576,6 +661,8 @@ int zmq::curve_server_t::receive_and_process_zap_reply ()
         if (rc == -1)
             break;
         if ((msg [i].flags () & msg_t::more) == (i < 6? 0: msg_t::more)) {
+            //  Temporary support for security debugging
+            puts ("CURVE I: ZAP handler sent incomplete reply message");
             errno = EPROTO;
             rc = -1;
             break;
@@ -587,35 +674,49 @@ int zmq::curve_server_t::receive_and_process_zap_reply ()
 
     //  Address delimiter frame
     if (msg [0].size () > 0) {
-        rc = -1;
+        //  Temporary support for security debugging
+        puts ("CURVE I: ZAP handler sent malformed reply message");
         errno = EPROTO;
+        rc = -1;
         goto error;
     }
 
     //  Version frame
     if (msg [1].size () != 3 || memcmp (msg [1].data (), "1.0", 3)) {
-        rc = -1;
+        //  Temporary support for security debugging
+        puts ("CURVE I: ZAP handler sent bad version number");
         errno = EPROTO;
+        rc = -1;
         goto error;
     }
 
     //  Request id frame
     if (msg [2].size () != 1 || memcmp (msg [2].data (), "1", 1)) {
-        rc = -1;
+        //  Temporary support for security debugging
+        puts ("CURVE I: ZAP handler sent bad request ID");
         errno = EPROTO;
+        rc = -1;
         goto error;
     }
 
     //  Status code frame
-    if (msg [3].size () != 3 || memcmp (msg [3].data (), "200", 3)) {
-        rc = -1;
+    if (msg [3].size () != 3) {
+        //  Temporary support for security debugging
+        puts ("CURVE I: ZAP handler rejected client authentication");
         errno = EACCES;
+        rc = -1;
         goto error;
     }
 
+    //  Save status code
+    status_code.assign (static_cast <char *> (msg [3].data ()), 3);
+
+    //  Save user id
+    set_user_id (msg [5].data (), msg [5].size ());
+
     //  Process metadata frame
     rc = parse_metadata (static_cast <const unsigned char*> (msg [6].data ()),
-                         msg [6].size ());
+                         msg [6].size (), true);
 
 error:
     for (int i = 0; i < 7; i++) {
