@@ -1,18 +1,27 @@
 /*
-    Copyright (c) 2007-2011 iMatix Corporation
-    Copyright (c) 2007-2011 Other contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
 
-    This file is part of 0MQ.
+    This file is part of libzmq, the ZeroMQ core engine in C++.
 
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -30,22 +39,27 @@
 
 #include "io_thread.hpp"
 #include "pgm_sender.hpp"
+#include "session_base.hpp"
 #include "err.hpp"
 #include "wire.hpp"
 #include "stdint.hpp"
 
-zmq::pgm_sender_t::pgm_sender_t (io_thread_t *parent_, 
+zmq::pgm_sender_t::pgm_sender_t (io_thread_t *parent_,
       const options_t &options_) :
     io_object_t (parent_),
     has_tx_timer (false),
     has_rx_timer (false),
+    session (NULL),
     encoder (0),
+    more_flag (false),
     pgm_socket (false, options_),
     options (options_),
     out_buffer (NULL),
     out_buffer_size (0),
     write_size (0)
 {
+    int rc = msg.init ();
+    errno_assert (rc == 0);
 }
 
 int zmq::pgm_sender_t::init (bool udp_encapsulation_, const char *network_)
@@ -61,15 +75,15 @@ int zmq::pgm_sender_t::init (bool udp_encapsulation_, const char *network_)
     return rc;
 }
 
-void zmq::pgm_sender_t::plug (io_thread_t *io_thread_, i_inout *inout_)
+void zmq::pgm_sender_t::plug (io_thread_t *io_thread_, session_base_t *session_)
 {
     //  Alocate 2 fds for PGM socket.
-    int downlink_socket_fd = 0;
-    int uplink_socket_fd = 0;
-    int rdata_notify_fd = 0;
-    int pending_notify_fd = 0;
+    fd_t downlink_socket_fd = retired_fd;
+    fd_t uplink_socket_fd = retired_fd;
+    fd_t rdata_notify_fd = retired_fd;
+    fd_t pending_notify_fd = retired_fd;
 
-    encoder.set_inout (inout_);
+    session = session_;
 
     //  Fill fds from PGM transport and add them to the poller.
     pgm_socket.get_sender_fds (&downlink_socket_fd, &uplink_socket_fd,
@@ -77,7 +91,7 @@ void zmq::pgm_sender_t::plug (io_thread_t *io_thread_, i_inout *inout_)
 
     handle = add_fd (downlink_socket_fd);
     uplink_handle = add_fd (uplink_socket_fd);
-    rdata_notify_handle = add_fd (rdata_notify_fd);   
+    rdata_notify_handle = add_fd (rdata_notify_fd);
     pending_notify_handle = add_fd (pending_notify_fd);
 
     //  Set POLLIN. We wont never want to stop polling for uplink = we never
@@ -106,7 +120,7 @@ void zmq::pgm_sender_t::unplug ()
     rm_fd (uplink_handle);
     rm_fd (rdata_notify_handle);
     rm_fd (pending_notify_handle);
-    encoder.set_inout (NULL);
+    session = NULL;
 }
 
 void zmq::pgm_sender_t::terminate ()
@@ -115,19 +129,22 @@ void zmq::pgm_sender_t::terminate ()
     delete this;
 }
 
-void zmq::pgm_sender_t::activate_out ()
+void zmq::pgm_sender_t::restart_output ()
 {
     set_pollout (handle);
     out_event ();
 }
 
-void zmq::pgm_sender_t::activate_in ()
+void zmq::pgm_sender_t::restart_input ()
 {
     zmq_assert (false);
 }
 
 zmq::pgm_sender_t::~pgm_sender_t ()
 {
+    int rc = msg.close ();
+    errno_assert (rc == 0);
+
     if (out_buffer) {
         free (out_buffer);
         out_buffer = NULL;
@@ -152,27 +169,40 @@ void zmq::pgm_sender_t::in_event ()
 
 void zmq::pgm_sender_t::out_event ()
 {
-    //  POLLOUT event from send socket. If write buffer is empty, 
+    //  POLLOUT event from send socket. If write buffer is empty,
     //  try to read new data from the encoder.
     if (write_size == 0) {
 
-        //  First two bytes (sizeof uint16_t) are used to store message 
+        //  First two bytes (sizeof uint16_t) are used to store message
         //  offset in following steps. Note that by passing our buffer to
         //  the get data function we prevent it from returning its own buffer.
         unsigned char *bf = out_buffer + sizeof (uint16_t);
         size_t bfsz = out_buffer_size - sizeof (uint16_t);
-        int offset = -1;
-        encoder.get_data (&bf, &bfsz, &offset);
+        uint16_t offset = 0xffff;
+
+        size_t bytes = encoder.encode (&bf, bfsz);
+        while (bytes < bfsz) {
+            if (!more_flag && offset == 0xffff)
+                offset = static_cast <uint16_t> (bytes);
+            int rc = session->pull_msg (&msg);
+            if (rc == -1)
+                break;
+            more_flag = msg.flags () & msg_t::more;
+            encoder.load_msg (&msg);
+            bf = out_buffer + sizeof (uint16_t) + bytes;
+            bytes += encoder.encode (&bf, bfsz - bytes);
+        }
 
         //  If there are no data to write stop polling for output.
-        if (!bfsz) {
+        if (bytes == 0) {
             reset_pollout (handle);
             return;
         }
 
+        write_size = sizeof (uint16_t) + bytes;
+
         //  Put offset information in the buffer.
-        write_size = bfsz + sizeof (uint16_t);
-        put_uint16 (out_buffer, offset == -1 ? 0xffff : (uint16_t) offset);
+        put_uint16 (out_buffer, offset);
     }
 
     if (has_tx_timer) {
@@ -184,17 +214,18 @@ void zmq::pgm_sender_t::out_event ()
     size_t nbytes = pgm_socket.send (out_buffer, write_size);
 
     //  We can write either all data or 0 which means rate limit reached.
-    if (nbytes == write_size) {
+    if (nbytes == write_size)
         write_size = 0;
-    } else {
+    else {
         zmq_assert (nbytes == 0);
 
         if (errno == ENOMEM) {
             const long timeout = pgm_socket.get_tx_timeout ();
             add_timer (timeout, tx_timer_id);
             has_tx_timer = true;
-        } else
-            zmq_assert (errno == EBUSY);
+        }
+        else
+            errno_assert (errno == EBUSY);
     }
 }
 
@@ -204,10 +235,13 @@ void zmq::pgm_sender_t::timer_event (int token)
     if (token == rx_timer_id) {
         has_rx_timer = false;
         in_event ();
-    } else if (token == tx_timer_id) {
+    }
+    else
+    if (token == tx_timer_id) {
         has_tx_timer = false;
         out_event ();
-    } else
+    }
+    else
         zmq_assert (false);
 }
 
