@@ -1,17 +1,27 @@
 /*
-    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
 
-    This file is part of 0MQ.
+    This file is part of libzmq, the ZeroMQ core engine in C++.
 
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -31,11 +41,12 @@ zmq::router_t::router_t (class ctx_t *parent_, uint32_t tid_, int sid_) :
     more_in (false),
     current_out (NULL),
     more_out (false),
-    next_peer_id (generate_random ()),
+    next_rid (generate_random ()),
     mandatory (false),
     //  raw_sock functionality in ROUTER is deprecated
-    raw_sock (false),       
-    probe_router (false)
+    raw_sock (false),
+    probe_router (false),
+    handover (false)
 {
     options.type = ZMQ_ROUTER;
     options.recv_identity = true;
@@ -87,6 +98,12 @@ int zmq::router_t::xsetsockopt (int option_, const void *optval_,
     int value = is_int? *((int *) optval_): 0;
 
     switch (option_) {
+        case ZMQ_CONNECT_RID:
+            if (optval_ && optvallen_) {
+                connect_rid.assign ((char *) optval_, optvallen_);
+                return 0;
+            }
+            break;
         case ZMQ_ROUTER_RAW:
             if (is_int && value >= 0) {
                 raw_sock = (value != 0);
@@ -112,6 +129,13 @@ int zmq::router_t::xsetsockopt (int option_, const void *optval_,
             }
             break;
 
+        case ZMQ_ROUTER_HANDOVER:
+            if (is_int && value >= 0) {
+                handover = (value != 0);
+                return 0;
+            }
+            break;
+
         default:
             break;
     }
@@ -232,12 +256,16 @@ int zmq::router_t::xsend (msg_t *msg_)
         }
 
         bool ok = current_out->write (msg_);
-        if (unlikely (!ok))
-            current_out = NULL;
-        else
-        if (!more_out) {
-            current_out->flush ();
+        if (unlikely (!ok)) {
+            // Message failed to send - we must close it ourselves.
+            int rc = msg_->close ();
+            errno_assert (rc == 0);
             current_out = NULL;
+        } else {
+          if (!more_out) {
+              current_out->flush ();
+              current_out = NULL;
+          }
         }
     }
     else {
@@ -363,19 +391,35 @@ bool zmq::router_t::xhas_out ()
     return true;
 }
 
+zmq::blob_t zmq::router_t::get_credential () const
+{
+    return fq.get_credential ();
+}
+
 bool zmq::router_t::identify_peer (pipe_t *pipe_)
 {
     msg_t msg;
     blob_t identity;
     bool ok;
 
+    if (connect_rid.length()) {
+        identity = blob_t ((unsigned char*) connect_rid.c_str (),
+            connect_rid.length());
+        connect_rid.clear ();
+        outpipes_t::iterator it = outpipes.find (identity);
+        if (it != outpipes.end ())
+            zmq_assert(false); //  Not allowed to duplicate an existing rid
+    }
+    else
     if (options.raw_sock) { //  Always assign identity for raw-socket
         unsigned char buf [5];
         buf [0] = 0;
-        put_uint32 (buf + 1, next_peer_id++);
+        put_uint32 (buf + 1, next_rid++);
         identity = blob_t (buf, sizeof buf);
     }
-    else {
+    else
+    if (!options.raw_sock) {
+        //  Pick up handshake cases and also case where next identity is set
         msg.init ();
         ok = pipe_->read (&msg);
         if (!ok)
@@ -385,7 +429,7 @@ bool zmq::router_t::identify_peer (pipe_t *pipe_)
             //  Fall back on the auto-generation
             unsigned char buf [5];
             buf [0] = 0;
-            put_uint32 (buf + 1, next_peer_id++);
+            put_uint32 (buf + 1, next_rid++);
             identity = blob_t (buf, sizeof buf);
             msg.close ();
         }
@@ -394,9 +438,34 @@ bool zmq::router_t::identify_peer (pipe_t *pipe_)
             outpipes_t::iterator it = outpipes.find (identity);
             msg.close ();
 
-            //  Ignore peers with duplicate ID.
-            if (it != outpipes.end ())
-                return false;
+            if (it != outpipes.end ()) {
+                if (!handover)
+                    //  Ignore peers with duplicate ID
+                    return false;
+                else {
+                    //  We will allow the new connection to take over this
+                    //  identity. Temporarily assign a new identity to the
+                    //  existing pipe so we can terminate it asynchronously.
+                    unsigned char buf [5];
+                    buf [0] = 0;
+                    put_uint32 (buf + 1, next_rid++);
+                    blob_t new_identity = blob_t (buf, sizeof buf);
+
+                    it->second.pipe->set_identity (new_identity);
+                    outpipe_t existing_outpipe =
+                        {it->second.pipe, it->second.active};
+
+                    ok = outpipes.insert (outpipes_t::value_type (
+                        new_identity, existing_outpipe)).second;
+                    zmq_assert (ok);
+
+                    //  Remove the existing identity entry to allow the new
+                    //  connection to take the identity.
+                    outpipes.erase (it);
+
+                    existing_outpipe.pipe->terminate (true);
+                }
+            }
         }
     }
 
