@@ -1,17 +1,27 @@
 /*
-    Copyright (c) 2007-2013 Contributors as noted in the AUTHORS file
+    Copyright (c) 2007-2015 Contributors as noted in the AUTHORS file
 
-    This file is part of 0MQ.
+    This file is part of libzmq, the ZeroMQ core engine in C++.
 
-    0MQ is free software; you can redistribute it and/or modify it under
-    the terms of the GNU Lesser General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
+    libzmq is free software; you can redistribute it and/or modify it under
+    the terms of the GNU Lesser General Public License (LGPL) as published
+    by the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
-    0MQ is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
+    As a special exception, the Contributors give you permission to link
+    this library with independent modules to produce an executable,
+    regardless of the license terms of these independent modules, and to
+    copy and distribute the resulting executable under terms of your choice,
+    provided that you also meet, for each linked independent module, the
+    terms and conditions of the license of that module. An independent
+    module is a module which is not derived from or based on this library.
+    If you modify this library, you must extend this exception to your
+    version of the library.
+
+    libzmq is distributed in the hope that it will be useful, but WITHOUT
+    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+    FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
+    License for more details.
 
     You should have received a copy of the GNU Lesser General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
@@ -28,7 +38,7 @@ memcpy(zmq_msg_data(&msg), data, size + 1);
 int publicationsReceived = 0;
 bool isSubscribed = false;
 
-int main(int argc, char** argv) {
+int main(int, char**) {
     setup_test_environment();
     void* context = zmq_ctx_new();
     void* pubSocket;
@@ -37,21 +47,21 @@ int main(int argc, char** argv) {
     (pubSocket = zmq_socket(context, ZMQ_XPUB))         || printf("zmq_socket: %s\n", zmq_strerror(errno));
     (subSocket = zmq_socket(context, ZMQ_SUB))          || printf("zmq_socket: %s\n", zmq_strerror(errno));
     zmq_setsockopt(subSocket, ZMQ_SUBSCRIBE, "foo", 3)  && printf("zmq_setsockopt: %s\n",zmq_strerror(errno));
-  
+
     zmq_bind(pubSocket, "inproc://someInProcDescriptor") && printf("zmq_bind: %s\n", zmq_strerror(errno));
     //zmq_bind(pubSocket, "tcp://127.0.0.1:30010") && printf("zmq_bind: %s\n", zmq_strerror(errno));
-  
+
     int more;
     size_t more_size = sizeof(more);
     int iteration = 0;
-  
+
     while (1) {
         zmq_pollitem_t items [] = {
             { subSocket, 0, ZMQ_POLLIN, 0 }, // read publications
             { pubSocket, 0, ZMQ_POLLIN, 0 }, // read subscriptions
         };
         int rc = zmq_poll (items, 2, 100);
-    
+
         if (items [1].revents & ZMQ_POLLIN) {
             while (1) {
                 zmq_msg_t msg;
@@ -62,7 +72,7 @@ int main(int argc, char** argv) {
                 if (buffer[0] == 0) {
                     assert(isSubscribed);
                     isSubscribed = false;
-                } 
+                }
                 else {
                     assert(!isSubscribed);
                     isSubscribed = true;
@@ -83,7 +93,7 @@ int main(int argc, char** argv) {
                 zmq_msg_recv (&msg, subSocket, 0);
                 zmq_getsockopt (subSocket, ZMQ_RCVMORE, &more, &more_size);
                 zmq_msg_close (&msg);
-        
+
                 if (!more) {
                     publicationsReceived++;
                     break;      //  Last message part
@@ -100,7 +110,7 @@ int main(int argc, char** argv) {
         }
         if (iteration > 4 && rc == 0)
             break;
-        
+
         zmq_msg_t channelEnvlp;
         ZMQ_PREPARE_STRING(channelEnvlp, "foo", 3);
         zmq_msg_send (&channelEnvlp, pubSocket, ZMQ_SNDMORE) >= 0 || printf("zmq_msg_send: %s\n",zmq_strerror(errno));
@@ -118,7 +128,7 @@ int main(int argc, char** argv) {
 
     zmq_close(pubSocket) && printf("zmq_close: %s", zmq_strerror(errno));
     zmq_close(subSocket) && printf("zmq_close: %s", zmq_strerror(errno));
-  
+
     zmq_ctx_term(context);
     return 0;
 }
